import java.util.NoSuchElementException;

import org.svetovid.io.SvetovidReader;
import org.svetovid.io.SvetovidWriter;

/*
 * Ova klasa sluzi za ucitavanje i snimanje stabala. Nije potrebno znati je,
 * i bice data, prilikom izrade prakticnih zadataka.
 *
 * Ocekivani format fajla je sledeci:
 *
 * br
 * id leviId desniId vrednost (x br)
 *
 * Primer fajla je classic.txt
 */
public class StabloIOClassic implements StabloIO {

    @Override
    public Stablo readStablo(SvetovidReader in) {
        int br = in.readInt();
        Stablo stablo = null;
        for (int i = 0; i < br; i++) {
            int id = in.readInt();
            int leviId = in.readInt();
            int desniId = in.readInt();
            String vrednost = in.readLine();
            Stablo levi = leviId == -1 ? null : new Stablo(leviId);
            Stablo desni =desniId == -1 ? null : new Stablo(desniId);
            Stablo element = new Stablo(id, vrednost, levi, desni);
            stablo = insert(stablo, element);
        }
        return stablo;
    }

    private static Stablo insert(Stablo stablo, Stablo element) {
        if (stablo == null) {
            return element;
        }
        Stablo found = find(stablo, element.getId());
        if (found == null) {
            throw new NoSuchElementException("id: " + element.getId());
        }
        found.setVrednost(element.getVrednost());
        found.setLevi(element.getLevi());
        found.setDesni(element.getDesni());
        return stablo;
    }

    private static Stablo find(Stablo stablo, int id) {
        if (stablo == null) {
            return null;
        }
        if (stablo.getId() == id) {
            return stablo;
        }
        Stablo rezultat = find(stablo.getLevi(), id);
        if (rezultat != null) {
            return rezultat;
        }
        return find(stablo.getDesni(), id);
    }

    @Override
    public void printStablo(SvetovidWriter out, Stablo stablo) {
        int br = count(stablo);
        out.println(br);
        iterate(out, stablo);
    }

    private static int count(Stablo stablo) {
        if (stablo == null) {
            return 0;
        }
        return 1 + count(stablo.getLevi()) + count(stablo.getDesni());
    }

    private static void iterate(SvetovidWriter out, Stablo stablo) {
        if (stablo == null) {
            return;
        }
        int id = stablo.getId();
        int leviId = stablo.getLevi() == null ? -1 : stablo.getLevi().getId();
        int desniId = stablo.getDesni() == null ? -1 : stablo.getDesni().getId();
        String vrednost = stablo.getVrednost();
        out.println(id, leviId, desniId, vrednost);
        iterate(out, stablo.getLevi());
        iterate(out, stablo.getDesni());
    }
}
